package com.css.simulation.resource.common.controller;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.RedisParameter;
import api.common.util.HttpUtil;
import api.common.util.JsonUtil;
import api.common.util.ObjectUtil;
import com.css.simulation.resource.common.configuration.algPlatform.ClientDetail;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;

@Controller
@RequestMapping("/algPlatform")
public class AlgPlatformCtrl {

    private static final String TOKEN_KEY = "ALGPLATFORM:TOKEN";

    @Autowired
    RedisController redisController;

    @Resource
    ClientDetail clientDetail;

    @RequestMapping("/getAlgorithmList")
    @ResponseBody
    @SneakyThrows
    public ResponseBodyVO getAlgorithmList(@RequestBody String param) {
        //取得token
        String token = getToken();
        if (ObjectUtil.isNull(token)) {
            return new ResponseBodyVO(ResponseBodyVO.Response.SERVER_FAILURE, "获取算法平台token失败！");
        }
        String algorithmListUri = clientDetail.getAlgorithmListUri() + "?access_token=" + token + param;
        String result = HttpUtil.get(HttpUtil.getHttpClient(), HttpUtil.getRequestConfig(), algorithmListUri);
        ResponseBodyVO responseBodyVO = new ResponseBodyVO(ResponseBodyVO.Response.SUCCESS, "请求成功！", result);
        return responseBodyVO;
    }

    @RequestMapping("/selectAlgorithmNameByAlgorithmId")
    @ResponseBody
    @SneakyThrows
    public ResponseBodyVO<String> selectAlgorithmNameByAlgorithmId(@RequestBody String algorithmId) {
        //取得token
        String token = getToken();
        if (ObjectUtil.isNull(token)) {
            return new ResponseBodyVO<>(ResponseBodyVO.Response.SERVER_FAILURE, "AlgPlatformCtrl--selectAlgorithmNameByAlgorithmId 获取算法平台token失败。");
        }
        String algorithmListUri = clientDetail.getAlgorithmListUri()
                + "?algorithmId=" + algorithmId
                + "&page=1&size=10&sort=algorithmId-desc&access_token=" + token;
        String httpResponseString = HttpUtil.get(HttpUtil.getHttpClient(), HttpUtil.getRequestConfig(), algorithmListUri);
        String algorithmName = new ObjectMapper().readTree(httpResponseString).path("data").path("content").get(0).path("algorithmName").asText();

        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, "请求成功！", algorithmName);
    }

    private String getToken() throws Exception {
        RedisParameter redisParameter = new RedisParameter();
        redisParameter.setKey(TOKEN_KEY);
        ResponseBodyVO<String> responseBodyVO = redisController.get(redisParameter);
        String token = responseBodyVO.getInfo();
        if (ObjectUtil.isNull(token)) {//token失效重新获取
            String tokenUri = clientDetail.getTokenUri();
            String appid = clientDetail.getAppid();
            String secret = clientDetail.getSecret();
            String URI = tokenUri + "&appid=" + appid + "&secret=" + secret;
            String result = HttpUtil.get(HttpUtil.getHttpClient(), HttpUtil.getRequestConfig(), URI);
            JsonNode jsonNode = JsonUtil.readTree(result);
            JsonNode dataNode = jsonNode.path("data");
            if (ObjectUtil.isNotNull(dataNode)) {
                token = dataNode.path("access_token").asText();
                if (ObjectUtil.isNotNull(token)) {
                    int expiresSec = dataNode.path("expires_in_sec").asInt();
                    redisParameter.setMinutes(expiresSec / 60);
                    redisParameter.setValue(token);
                    redisController.set(redisParameter);
                }
            }
        }
        return token;
    }

}
