package com.css.simulation.resource.scene.controller;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.scene.SceneImportParam;
import api.common.pojo.po.system.SceneImportPO;
import com.css.simulation.resource.scene.common.oauth.AuthorizationHolder;
import com.css.simulation.resource.scene.feign.SceneService;
import com.css.simulation.resource.scene.service.SceneImportService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

@Controller
@Slf4j
@RequestMapping("/sceneImport")
public class SceneImportCtrl {

    @Resource
    SceneImportService sceneImportService;

    @Resource
    SceneService sceneService;

    /**
     * 先上传到 minio 然后解析
     */
    @RequestMapping("/startImport")
    @ResponseBody
    public ResponseBodyVO<String> startImport(@RequestBody SceneImportPO po, HttpServletRequest request) {
        try {
            //* -------------------------------- 路径校验 --------------------------------
            Boolean directory = sceneImportService.checkDirectory(po);
            if (!directory) {
                po.setErrorMessage("路径错误: " + po.getDataDirectory());
                po.setStatus(DictConstants.SCENE_IMPORT_STATUS_0);
                sceneImportService.updateStatus(po);
                return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            //* -------------------------------- 上传到 minio --------------------------------
            po.setStatus(DictConstants.SCENE_IMPORT_STATUS_1);
            sceneImportService.updateStatus(po);
            log.info("上传到minio " + po);
            Boolean sceneType = sceneImportService.startImport(po); // 上传
            if (!sceneType) {
                po.setErrorMessage("场景类型错误！");
                po.setStatus(DictConstants.SCENE_IMPORT_STATUS_0);
                sceneImportService.updateStatus(po);
                return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE);
            }
            po.setStatus(DictConstants.SCENE_IMPORT_STATUS_2);
            sceneImportService.updateStatus(po);
        } catch (Exception e) {
            e.printStackTrace();
            po.setStatus(DictConstants.SCENE_IMPORT_STATUS_0);
            po.setErrorMessage("上传失败");
            sceneImportService.updateStatus(po);
            return new ResponseBodyVO<>(ResponseBodyVO.Response.SERVER_FAILURE);
        }
        //* -------------------------------- 解析 --------------------------------
        SceneImportParam param = new SceneImportParam();
        param.setId(po.getId());
        param.setMinioDirectory(po.getDataDirectory());
        String authorization = request.getHeader("Authorization");
        AuthorizationHolder.setAuthorization(authorization);
        switch (po.getSceneType()) {
            case DictConstants.SCENE_NATURAL:
                sceneService.saveSceneNaturalBatch(param);  //自然驾驶
                break;
            case DictConstants.SCENE_STANDARD:  // 标准法规
                sceneService.saveStandardsRegulationsBatch(param);
                break;
            case DictConstants.SCENE_ACCIDENT:  // 交通事故
                sceneService.saveSceneAccidentBatch(param);
        }
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS);
    }
}
