package com.css.simulation.resource.scheduler.mapper;


import com.css.simulation.resource.scheduler.entity.ProjectEntity;
import org.apache.ibatis.annotations.*;
import org.apache.ibatis.type.JdbcType;

import java.sql.Timestamp;
import java.util.List;

@Mapper
public interface AutoSubProjectMapper {

    @Results(id = "project", value = {
            @Result(column = "id", property = "id", jdbcType = JdbcType.VARCHAR),
            @Result(column = "scene", property = "scenePackageId", jdbcType = JdbcType.VARCHAR),
            @Result(column = "create_user_id", property = "createUserId", jdbcType = JdbcType.VARCHAR),
            @Result(column = "parallelism", property = "parallelism", jdbcType = JdbcType.VARCHAR),
            @Result(column = "project_type", property = "projectType", jdbcType = JdbcType.VARCHAR),
            @Result(column = "is_choice_gpu", property = "isChoiceGpu", jdbcType = JdbcType.VARCHAR),
            @Result(column = "max_simulation_time", property = "maxSimulationTime", jdbcType = JdbcType.VARCHAR)
    })
    @Select("select sas.id,\n" +
            "       sap.scene,\n" +
            "       sas.create_user_id,\n" +
            "       '2' project_type,\n" +
            "       sap.is_choice_gpu,\n" +
            "       sap.max_simulation_time\n" +
            "from simulation_automatic_subproject sas\n" +
            "         left join simulation_automatic_project sap on sas.parent_id = sap.id\n" +
            "where sas.id = #{projectId}")
    ProjectEntity selectById(@Param("projectId") String projectId);

    @ResultMap("project")
    @Select("select sas.id, sap.scene, sas.create_user_id, sap.parallelism, '2' project_type\n" +
            "from simulation_automatic_subproject sas\n" +
            "         left join simulation_automatic_project sap on sas.parent_id = sap.id\n" +
            "where sas.is_deleted = '0'\n" +
            "  and sas.now_run_state = #{nowRunState}")
    List<ProjectEntity> selectByNowRunState(@Param("nowRunState") String nowRunState);

    @Select("select create_user_id\n" +
            "from simulation_automatic_subproject\n" +
            "where id = #{subProjectId}")
    String selectCreateUserById(@Param("subProjectId") String subProjectId);


    @Update("update simulation_automatic_subproject\n" +
            "set now_run_state  = #{state},\n" +
            "    finish_time = #{finishTime}\n" +
            "where id = #{id}")
    void updateProjectState(@Param("id") String id, @Param("state") String state, @Param("finishTime") Timestamp finishTime);

    @Select("select is_choice_gpu\n" +
            "from simulation_automatic_subproject\n" +
            "where id = #{projectId}")
    String selectIsChoiceGpuById(@Param("projectId") String projectId);

}
