package com.css.simulation.resource.scheduler.service;

import com.css.simulation.resource.scheduler.configuration.redis.CustomRedisClient;
import com.css.simulation.resource.scheduler.entity.PrefixEntity;
import com.css.simulation.resource.scheduler.entity.ProjectEntity;
import com.css.simulation.resource.scheduler.entity.TaskEntity;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.util.ProjectUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
@Slf4j
public class TaskService {
    @Resource
    private TaskManager taskManager;
    @Resource
    private TaskMapper taskMapper;
    @Resource
    private ProjectUtil projectUtil;
    @Resource
    private CustomRedisClient customRedisClient;


    // -------------------------------- Comment --------------------------------

    @SneakyThrows
    public void taskState(String taskId, String state, String podName) {
        String lockName = "taskId:" + taskId + ":state:" + state + ":pod-name:" + podName;
        customRedisClient.lock(lockName, 1L, 30 * 60L);
        try {
            TaskEntity taskEntity = taskMapper.selectById(taskId);
            String projectId = taskEntity.getPId(); // 项目 id
            ProjectEntity projectEntity = projectUtil.getProjectByProjectId(projectId);
            String projectType = projectEntity.getProjectType();  // 项目类型
            String maxSimulationTime = projectEntity.getMaxSimulationTime();  // 项目类型
            String userId = taskEntity.getCreateUserId();   // 用户 id
            PrefixEntity redisPrefix = projectUtil.getRedisPrefixByUserIdAndProjectIdAndTaskId(userId, projectId, taskId);  // 项目前缀
            boolean projectCompleted = taskManager.isProjectCompleted(redisPrefix, projectId, projectType, maxSimulationTime, taskId, state, podName);
            if (projectCompleted) {
                log.info("项目 {} 开始打分。", projectId);
                taskManager.score(redisPrefix.getProjectRunningKey(), userId, projectId, projectType);
                log.info("项目 {} 计算评价等级。", projectId);
                taskManager.evaluationLevel(projectId);
                log.info("项目 {} 开始释放资源。", projectId);
                taskManager.done(redisPrefix, projectId, projectType);
                log.info("项目 {} 运行结束。", projectId);
            }
        } finally {
            customRedisClient.unlock(lockName);
        }

    }


    public Boolean taskConfirm(String taskId) {
        return taskManager.taskConfirm(taskId);
    }

    public void taskTick(String taskId) {
        taskManager.taskTick(taskId);
    }


}



