package com.css.simulation.resource.algorithm.controller;


import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.LogConstants;
import api.common.pojo.param.algorithm.AlgorithmParameter;
import com.css.simulation.resource.algorithm.service.AlgorithmService;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Map;

/**
 * 算法库模块
 */
@Controller
@RequestMapping(value = "/algorithm",name = LogConstants.MODULE_ALGORITH)
public class AlgorithmController {

    @Resource
    private AlgorithmService algorithmService;

    //* -------------------------------- Comm删除算法ent --------------------------------
    @RequestMapping("deleteByid")
    @ResponseBody
    public ResponseBodyVO<String> deleteById(@RequestBody AlgorithmParameter param){
        return algorithmService.deleteById(param);
    }
    //* -------------------------------- Comment --------------------------------


    /**
     * 新增/修改算法
     */
    @RequestMapping("addOrUpdate")
    @ResponseBody
    public ResponseBodyVO addOrUpdate(@RequestBody AlgorithmParameter param) throws IOException {
        return algorithmService.addOrUpdate(param);
    }


    /**
     * 查询算法列表-私有导入和私有仓库页面
     * @param param
     * @return
     */
    @RequestMapping("selectAlgorithmList")
    @ResponseBody
    public ResponseBodyVO selectAlgorithmList(@RequestBody AlgorithmParameter param){
        return algorithmService.selectAlgorithmList(param);
    }

    /**
     * 查询算法列表-算法平台页面
     * @param param
     * @return
     */
    @RequestMapping("selectAlgoPlatformList")
    @ResponseBody
    public ResponseBodyVO selectAlgoPlatformList(@RequestBody AlgorithmParameter param){
        return algorithmService.selectAlgoPlatformList(param);
    }


    /**
     * 查询算法列表-公有页面
     * @param param
     * @return
     */
    @RequestMapping("selectSharedAlgorithmList")
    @ResponseBody
    public ResponseBodyVO selectSharedAlgorithmList(@RequestBody AlgorithmParameter param){
        return algorithmService.selectSharedAlgorithmList(param);
    }


    /**
     * 删除算法前校验
     */
    @RequestMapping("deleteCheck")
    @ResponseBody
    public ResponseBodyVO<Map<String,String>> deleteCheck(@RequestBody AlgorithmParameter param){
        return algorithmService.deleteCheck(param);
    }


    /**
     * 分享算法
     * @return
     */
    @RequestMapping("shareAlgorithm")
    @ResponseBody
    public ResponseBodyVO shareAlgorithm(@RequestBody AlgorithmParameter param){
        return algorithmService.shareAlgorithm(param);
    }


    /**
     * 查看算法详情
     * @param param
     * @return
     */
    @RequestMapping("selectDetailsById")
    @ResponseBody
    public ResponseBodyVO selectDetailsById(@RequestBody AlgorithmParameter param){
        String algorithmId = param.getId();
        return algorithmService.selectDetailsById(algorithmId);
    }

    /**
     * git测试连接
     */
    @RequestMapping("testConnection")
    @ResponseBody
    public ResponseBodyVO<String> testConnection(@RequestBody AlgorithmParameter param){
        return algorithmService.testConnection(param);
    }

    /**
     * 根据git仓库地址获取版本号
     */
    @RequestMapping("getGitVersion")
    @ResponseBody
    public ResponseBodyVO<String> getGitVersion(@RequestBody AlgorithmParameter param) {
        String id = param.getId();
        String gitVersion = algorithmService.getGitVersion(id);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS,"请求成功。", gitVersion);
    }

}
