package com.css.simulation.resource.model.service;

import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.model.CameraParam;
import api.common.pojo.po.model.CameraPO;
import api.common.pojo.vo.model.CameraVO;
import api.common.util.LogUtil;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.model.mapper.CameraMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.util.List;

@Service
public class CameraService {

    @Resource
    CameraMapper cameraMapper;

    public List<CameraVO> getCameraList(CameraParam cameraParam) {
        cameraParam.setCreateUserId(AuthUtil.getCurrentUserId());
        cameraParam.setShare(null);
        return cameraMapper.getCameraList(cameraParam);
    }

    public List<CameraVO> getMyCameraList(CameraParam cameraParam) {
        cameraParam.setCreateUserId(AuthUtil.getCurrentUserId());
//        cameraParam.setShare(DictConstants.NO);
        cameraParam.setShare(null);
        List<CameraVO> cameraList = cameraMapper.getCameraList(cameraParam);
        cameraList.forEach(camera -> {
            if ("1".equals(camera.getShare())) {
                camera.setSensorName(camera.getSensorName() + "（公有）");
            }
        });
        return cameraList;
    }

    public CameraVO getCameraInfo(CameraParam cameraParam) {
        return cameraMapper.getCameraInfo(cameraParam);
    }


    public CameraPO saveCamera(CameraPO cameraPO) {
        //常规字段赋值
        String currentUserId = AuthUtil.getCurrentUserId();
        Timestamp currentTime = TimeUtil.getNowForMysql();
        cameraPO.setCreateUserId(currentUserId);
        cameraPO.setCreateTime(currentTime);
        cameraPO.setModifyUserId(currentUserId);
        cameraPO.setModifyTime(currentTime);
        cameraPO.setIsDeleted(DictConstants.NO);
        cameraPO.setShare(DictConstants.NO);//私有
        //名称校验
        List<CameraVO> list = cameraMapper.checkCameraName(cameraPO);
        if (ObjectUtil.isNotNull(list)) {
            cameraPO.setId(null);
            return cameraPO;
        }
        String id = cameraPO.getId();
        if (ObjectUtil.isNull(id)) {//新增
            cameraPO.setId(StringUtil.getRandomUUID());
            cameraPO.setSensorCode(StringUtil.getRandomCode());
            cameraMapper.insert(cameraPO);
            LogUtil.insert();
        } else {//修改
            cameraMapper.update(cameraPO);
            LogUtil.update();
        }
        return cameraPO;
    }

    public CameraPO shareCamera(CameraPO cameraPO) {
        //调用保存
        saveCamera(cameraPO);
        cameraPO.setShare(DictConstants.YES);//转公有
        cameraPO.setCreateUserId(null);
        //名称校验
        List<CameraVO> list = cameraMapper.checkCameraName(cameraPO);
        if (ObjectUtil.isNotNull(list)) {
            String currentUserId = AuthUtil.getCurrentUserId();
            Timestamp currentTime = TimeUtil.getNowForMysql();
            cameraPO.setId(list.get(0).getId());
            cameraPO.setModifyUserId(currentUserId);
            cameraPO.setModifyTime(currentTime);
            cameraMapper.update(cameraPO);
            LogUtil.update();
        } else {
            //常规字段赋值
            String currentUserId = AuthUtil.getCurrentUserId();
            Timestamp currentTime = TimeUtil.getNowForMysql();
            cameraPO.setCreateUserId(currentUserId);
            cameraPO.setCreateTime(currentTime);
            cameraPO.setModifyUserId(currentUserId);
            cameraPO.setModifyTime(currentTime);
            cameraPO.setIsDeleted(DictConstants.NO);
            cameraPO.setId(StringUtil.getRandomUUID());
            cameraPO.setSensorCode(StringUtil.getRandomCode());
            cameraMapper.insert(cameraPO);
        }
        LogUtil.share();
        return cameraPO;
    }

    public void delCameraById(CameraPO cameraPO) {
        cameraPO.setIsDeleted(DictConstants.YES);
        cameraPO.setModifyUserId(AuthUtil.getCurrentUserId());
        cameraPO.setModifyTime(TimeUtil.getNowForMysql());
        cameraMapper.delCameraById(cameraPO);
        LogUtil.delete();
    }
}
