package com.css.simulation.resource.model.service;

import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.model.GpsParam;
import api.common.pojo.po.model.GpsPO;
import api.common.pojo.vo.model.GpsVO;
import api.common.util.LogUtil;
import api.common.util.ObjectUtil;
import api.common.util.StringUtil;
import api.common.util.TimeUtil;
import com.css.simulation.resource.common.utils.AuthUtil;
import com.css.simulation.resource.common.utils.PoUtil;
import com.css.simulation.resource.model.mapper.GpsMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.util.List;

@Service
public class GpsService {

    @Resource
    private GpsMapper gpsMapper;

    //* -------------------------------- 删除 GPS --------------------------------

    public void delGpsById(GpsPO gpsPO) {
        PoUtil.initDelPo(gpsPO);
        gpsMapper.delGpsById(gpsPO);
        LogUtil.delete();
    }
    //* -------------------------------- Comment --------------------------------

    public List<GpsVO> getGpsList(GpsParam gpsParam) {
        gpsParam.setCreateUserId(AuthUtil.getCurrentUserId());
        gpsParam.setShare(null);
        return gpsMapper.getGpsList(gpsParam);
    }

    public List<GpsVO> getMyGpsList(GpsParam gpsParam) {
        gpsParam.setCreateUserId(AuthUtil.getCurrentUserId());
        gpsParam.setShare(null);
        List<GpsVO> gpsList = gpsMapper.getGpsList(gpsParam);
        gpsList.forEach(gps-> {
            if("1".equals(gps.getShare()) ){
                gps.setSensorName(gps.getSensorName()+"（公有）");
            }
        });
        return gpsList;
    }

    public GpsVO getGpsInfo(GpsParam gpsParam) {
        return gpsMapper.getGpsInfo(gpsParam);
    }

    public GpsPO saveGps(GpsPO gpsPO) {
        //常规字段赋值
        PoUtil.initAddPo(gpsPO);
        gpsPO.setShare(DictConstants.NO);//私有
        //名称校验
        List<GpsVO> list = gpsMapper.checkGpsName(gpsPO);
        if(ObjectUtil.isNotNull(list)){
            gpsPO.setId(null);
            return gpsPO;
        }
        String id = gpsPO.getId();
        if(ObjectUtil.isNull(id)){//新增
            gpsPO.setId(StringUtil.getRandomUUID());
            gpsPO.setSensorCode(StringUtil.getRandomCode());
            gpsMapper.insert(gpsPO);
            LogUtil.insert();
        }else{//修改
            gpsMapper.update(gpsPO);
            LogUtil.update();
        }
        return gpsPO;
    }

    public GpsPO shareGps(GpsPO gpsPO) {
        //调用保存
        saveGps(gpsPO);
        gpsPO.setShare(DictConstants.YES);//转公有
        gpsPO.setCreateUserId(null);
        //名称校验
        List<GpsVO> list = gpsMapper.checkGpsName(gpsPO);
        if(ObjectUtil.isNotNull(list)){
            String currentUserId = AuthUtil.getCurrentUserId();
            Timestamp currentTime = TimeUtil.getNowForMysql();
            gpsPO.setId(list.get(0).getId());
            gpsPO.setModifyUserId(currentUserId);
            gpsPO.setModifyTime(currentTime);
            gpsMapper.update(gpsPO);
            LogUtil.update();
        }else {
            //常规字段赋值
            PoUtil.initAddPo(gpsPO);
            gpsPO.setId(StringUtil.getRandomUUID());
            gpsPO.setSensorCode(StringUtil.getRandomCode());
            gpsMapper.insert(gpsPO);
        }
        LogUtil.share();
        return gpsPO;
    }

}
