package com.css.simulation.resource.project.ctrl;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.project.DictParam;
import api.common.pojo.param.project.SimulationManualProjectParam;
import api.common.pojo.vo.project.*;
import api.common.util.StringUtil;
import com.css.simulation.resource.project.service.SimulationProjectService;
import com.css.simulation.resource.system.service.ClusterService;
import com.github.pagehelper.PageInfo;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * 工作台模块--项目运行
 */
@RestController
@RequestMapping("/simulationProject")
public class SimulationProjectCtrl {

    @Resource
    private SimulationProjectService service;
    @Resource
    private ClusterService clusterService;

    //* -------------------------------- 项目运行 --------------------------------

    /**
     * 1. 手动项目：第一次运行项目
     * 2. 手动项目：重新运行项目
     */
    @RequestMapping("updateProjectNowRunState")
    public ResponseBodyVO<String> updateProjectNowRunState(@RequestBody SimulationManualProjectParam param) {
        return service.updateProjectNowRunState(param);
    }

    /**
     * 修改自动运行状态
     */
    @RequestMapping("updateAutomaticRunState")
    public ResponseBodyVO<String> updateAutomaticRunState(@RequestBody SimulationManualProjectParam param) {
        return service.updateAutomaticRunState(param);
    }

    /**
     * 手动运行自动项目
     */
    @RequestMapping("createAutomaticSubProject")
    public ResponseBodyVO<String> createAutomaticSubProject(@RequestBody SimulationManualProjectParam param) {
        return service.createAutomaticSubProject(param);
    }

    //* -------------------------------- Comment --------------------------------


    /**
     * 导出测试报告
     */
    @RequestMapping("exportProjectReportById")
    public void exportProjectReportById(@RequestBody SimulationManualProjectParam param) {
        service.exportProjectReport(param);
    }

    //* -------------------------------- Comment --------------------------------

    /**
     * 创建/修改工作
     */
    @RequestMapping("addOrUpdateProject")
    public ResponseBodyVO<String> addOrUpdateProject(@RequestBody SimulationManualProjectParam param) {
        return service.addOrUpdateProject(param);
    }

    /**
     * 查询项目列表
     */
    @RequestMapping("selectProject")
    public ResponseBodyVO<PageInfo<SimulationManualProjectVo>> selectProject(@RequestBody SimulationManualProjectParam param) {
        return service.selectProject(param);
    }

    /**
     * 根据id查询
     */
    @RequestMapping("selectProjectById")
    public ResponseBodyVO<SimulationManualProjectSingleVo> selectProjectById(@RequestBody SimulationManualProjectParam param) {
        return service.selectProjectById(param);
    }

    /**
     * 删除工作（支持批量删除）
     */
    @RequestMapping("deleteProjectByids")
    public ResponseBodyVO<String> deleteProjectByids(@RequestBody SimulationManualProjectParam param) {
        return service.deleteProjectByids(param);
    }


    /**
     * 获取工作详情
     */
    @RequestMapping("selectProjectDetailsById")
    public ResponseBodyVO<ProjectDetailsVo> selectProjectDetailsById(@RequestBody SimulationManualProjectParam param) {
        return service.selectProjectDetailsById(param);
    }

    /**
     * 获取测试报告详情
     */
    @RequestMapping("selectProjectReportById")
    public ResponseBodyVO<ProjectReportVo> selectProjectReportById(@RequestBody SimulationManualProjectParam param) {
        return service.selectProjectReportById(param);
    }

    /**
     * 获取任务详情信息
     */
    @RequestMapping("selectProjectTaskById")
    public ResponseBodyVO<ProjectTaskDetailsVo> selectProjectTaskById(@RequestBody SimulationManualProjectParam param) {
        return service.selectProjectTaskById(param);
    }

    /**
     * 获取添加工作时需要的下拉选项
     * 0. 算法列表
     * 1. 车辆列表
     * 2. 场景列表
     */
    @RequestMapping("selectDropDownByType")
    public ResponseBodyVO<List<DropDownTypeVo>> selectDropDownByType(@RequestBody SimulationManualProjectParam param) {
        return service.selectDropDownByType(param);
    }

    /**
     * 获取添加工作时需要的下拉选项
     * 0. 算法列表
     * 1. 车辆列表
     * 2. 场景列表
     */
    @RequestMapping("selectDropDownByTypeNew")
    public ResponseBodyVO<List<DropDownTypeNewVo>> selectDropDownByTypeNew(@RequestBody SimulationManualProjectParam param) {
        return service.selectDropDownByTypeNew(param);
    }


    /**
     * @return 人员可用最大并行度
     */
    @RequestMapping("selectMaxParallelism")
    public ResponseBodyVO<String> selectMaxParallelism() {
        Integer licenseNum = clusterService.getLicenseNum();
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, null, licenseNum + "");
    }

    /**
     * 根据字典类型和
     *
     * @param param 参数
     * @return name
     */
    @RequestMapping("getDictName")
    public ResponseBodyVO<String> getDictName(@RequestBody DictParam param) {
        String dictName = service.getDictName(param.getType(), param.getCode());
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, null, dictName);

    }


    /**
     * 解析车辆坐标文件
     */
    @RequestMapping("analysisVehicleCoord")
    public ResponseBodyVO<String> analysisVehicleCoord(){
        return service.analysisVehicleCoord();
    }


    /**
     * 查询工作任务列表
     */
    @RequestMapping("selectProjectTaskList")
    public ResponseBodyVO<PageInfo<ManualProjectTaskVo>> selectProjectTaskList(@RequestBody SimulationManualProjectParam param) {
        return service.selectProjectTaskList(param);
    }


    /**
     * 生成保存评测等级（任务结果返回时调用）
     *
     * @return 响应对象
     */
    @RequestMapping("saveEvaluationLevel")
    public ResponseBodyVO<String> saveEvaluationLevel(@RequestBody SimulationManualProjectParam simulationManualProjectParam) {
        String id = simulationManualProjectParam.getId();
        if (StringUtil.isEmpty(id)) {
            return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE, "id 不能为空", "");
        } else {
            return service.saveEvaluationLevel(id);
        }

    }

    /**
     * 保存仿真结果到数据库（任务结果返回时调用）
     */
    @RequestMapping("saveTaskResult")
    public ResponseBodyVO saveTaskResult(@RequestBody SimulationManualProjectParam param) {
        return service.saveTaskResult(param);
    }

    /**
     * 导出工作任务文件包
     */
    @RequestMapping("exportProjectTaskFileById")
    public void exportProjectTaskFileById(@RequestBody SimulationManualProjectParam param) {
        service.exportProjectTaskFileById(param);
    }

    /**
     * 同时导出工作报告和任务包
     */
    @RequestMapping("exportProjectReportAndTaskFileById")
    public void exportProjectReportAndTaskFileById(@RequestBody SimulationManualProjectParam param) {
        service.exportProjectReportAndTaskFileById(param);
    }


    /**
     * 添加编辑自动项目
     */
    @RequestMapping("addOrUpdateAutomaticProject")
    public ResponseBodyVO addOrUpdateAutomaticProject(@RequestBody SimulationManualProjectParam param) {
        return service.addOrUpdateAutomaticProject(param);
    }


    /**
     * 删除自动运行工作（支持批量删除）
     */
    @RequestMapping("deleteAutomaticProjectByids")
    public ResponseBodyVO<String> deleteAutomaticProjectByids(@RequestBody SimulationManualProjectParam param) {
        return service.deleteAutomaticProjectByids(param);
    }


    ;

    /**
     * 查询自动运行项目列表
     */
    @RequestMapping("selectAutomaticProject")
    public ResponseBodyVO<PageInfo<SimulationManualProjectVo>> selectAutomaticProject(@RequestBody SimulationManualProjectParam param) {
        return service.selectAutomaticProject(param);
    }


    /**
     * 删除自动运行子项目（支持批量删除）
     */
    @RequestMapping("deleteAutomaticSubProjectByIds")
    public ResponseBodyVO deleteAutomaticSubProjectByIds(@RequestBody SimulationManualProjectParam param) {

        return service.deleteAutomaticSubProjectByIds(param);
    }

    /**
     * 查询自动运行子工作信息
     */
    @RequestMapping("selectSubProjectInfo")
    public ResponseBodyVO<AutomaticProjectVo> selectSubProjectInfo(@RequestBody SimulationManualProjectParam param) {
        return service.selectSubProjectInfo(param);
    }


    /**
     * 查询自动运行子工作列表
     */
    @RequestMapping("selectSubProjectList")
    public ResponseBodyVO<PageInfo<SimulationManualProjectVo>> selectSubProjectList(@RequestBody SimulationManualProjectParam param) {
        return service.selectSubProjectList(param);
    }

    /**
     * 根据id查询自动运行项目信息
     */
    @RequestMapping("selectAutomaticProjectById")
    public ResponseBodyVO<SimulationManualProjectSingleVo> selectAutomaticProjectById(@RequestBody SimulationManualProjectParam param) {
        return service.selectAutomaticProjectById(param);
    }

    /**
     * 修改自动运行子工作运行状态
     */
    @RequestMapping("updateAutoProjectNowRunState")
    public ResponseBodyVO<String> updateAutoProjectNowRunState(@RequestBody SimulationManualProjectParam param) {
        return service.updateAutoProjectNowRunState(param);
    }


}
