package com.css.simulation.resource.scene.ctrl;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.constants.DictConstants;
import api.common.pojo.param.scene.SceneImportParam;
import api.common.pojo.param.scene.StandardsRegulationsParam;
import api.common.pojo.po.scene.StandardsRegulationsPO;
import api.common.pojo.po.system.SceneImportPO;
import api.common.pojo.vo.scene.StandardsRegulationsVO;
import api.common.util.ObjectUtil;
import com.css.simulation.resource.common.utils.PageUtil;
import com.css.simulation.resource.scene.service.StandardsRegulationsService;
import com.css.simulation.resource.system.service.SceneImportService;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

//标准法规
@RestController
@RequestMapping("/StandardsRegulations")
public class StandardsRegulationsController {

    @Resource
    private StandardsRegulationsService standardsRegulationsService;
    @Resource
    private SceneImportService sceneImportService;

    @PostMapping("/batchDeleteSceneStandards")
    public ResponseBodyVO<String> batchDeleteSceneStandards(@RequestBody StandardsRegulationsParam params) {
       List<String> sceneNameList = params.getSceneNameList();
        standardsRegulationsService.batchDeleteSceneStandards(sceneNameList);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS);
    }

    //查询列表
    @PostMapping("/queryStandardsRegulationsList")
    public ResponseBodyVO<PageInfo<StandardsRegulationsVO>> queryStandardsRegulationsList(@RequestBody StandardsRegulationsParam params) {
        PageUtil.setPageInfo(params);
        List<StandardsRegulationsPO> list = standardsRegulationsService.queryStandardsRegulationsList(params);
        PageInfo<StandardsRegulationsPO> objectPageInfo = new PageInfo<>(list);
        List<StandardsRegulationsVO> list1 = new ArrayList<>();
        for (StandardsRegulationsPO po : list) {
            StandardsRegulationsVO vo = new StandardsRegulationsVO();
            BeanUtils.copyProperties(po, vo);
            list1.add(vo);
        }
        PageInfo<StandardsRegulationsVO> pageInfo = new PageInfo<>(list1);
        pageInfo.setTotal(objectPageInfo.getTotal());
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, pageInfo);
    }


    /**
     * 新增标准法规
     * @param params
     * @return
     */
    @PostMapping("/saveStandardsRegulations")
    public ResponseBodyVO<String> saveStandardsRegulations(@RequestBody StandardsRegulationsParam params) {
        return standardsRegulationsService.importBzFg(params);
    }


    /**
     * 删除
     */
    @PostMapping("/deleteStandardsRegulations")
    public ResponseBodyVO<String> deleteStandardsRegulations(@RequestBody StandardsRegulationsPO params) {
        if (ObjectUtil.isNull(params.getRegulationsId())) {
            return new ResponseBodyVO<>(ResponseBodyVO.Response.CLIENT_FAILURE, "参数场景Id不能为空，请检查");
        }
        standardsRegulationsService.deleteStandardsRegulations(params);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS);
    }

    /**
     * 测试批量导入
     */
    @PostMapping("/saveStandardsRegulationsBatch")
    public ResponseBodyVO<String> saveStandardsRegulationsBatch(@RequestBody SceneImportParam sceneImportParam) {
        SceneImportPO sceneImportPO = new SceneImportPO();
        sceneImportPO.setId(sceneImportParam.getId());
        sceneImportPO.setStatus(DictConstants.SCENE_IMPORT_STATUS_3);
        sceneImportService.updateStatus(sceneImportPO);
        // 解析参数
        sceneImportPO = standardsRegulationsService.importMinio(sceneImportParam);
        sceneImportService.updateInfo(sceneImportPO);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS);
    }


    /**
     * 查詢法规类型
     */
    @PostMapping("/queryRegulationType")
    public ResponseBodyVO<List<String>> queryRegulationType() {
        return standardsRegulationsService.queryRegulationType();
    }


    /**
     * 分享
     */
    @PostMapping("/instStandardsRegulation")
    public ResponseBodyVO<String> instStandardsRegulation(@RequestBody StandardsRegulationsPO po) {
        standardsRegulationsService.instStandardsRegulation(po);
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS);
    }

}