package com.css.simulation.resource.system.ctrl;

import api.common.pojo.common.ResponseBodyVO;
import api.common.pojo.param.system.DictParam;
import api.common.pojo.vo.system.DictVO;
import com.css.simulation.resource.system.service.DictService;
import com.fasterxml.jackson.core.JsonProcessingException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.stereotype.Controller;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;
import java.util.Map;

/**
 * 字典数据管理
 */
@Controller
@RequestMapping("/dict")
public class DictCtrl {

    @Autowired
    DictService dictService;

    /**
     * 通过字典类型dictTypes将字典数据刷新至缓存，当dictTypes值为all时，将刷新所有字典。
     *
     * @param param
     * @return
     */
    @RequestMapping("/refreshDictsByTypes")
    @ResponseBody
    @PreAuthorize("@AuthorityCheck.admin()")
    public ResponseBodyVO<String> refreshDictsByTypes(@RequestBody @Validated DictParam param) {
        return new ResponseBodyVO<>(ResponseBodyVO.Response.SUCCESS, dictService.refreshDicts(param));
    }

    /**
     * 通过字典类型dictTypes从缓存中查询字典数据lists
     *
     * @param param
     * @return
     * @throws JsonProcessingException
     */
    @RequestMapping("/getDictListsByTypes")
    @ResponseBody
    public ResponseBodyVO getDictListsByTypes(@RequestBody @Validated DictParam param) throws JsonProcessingException {
        ResponseBodyVO<Map<String, List<DictVO>>> response = new ResponseBodyVO<Map<String, List<DictVO>>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(dictService.getDictListsByTypes(param));
        return response;
    }

    /**
     * 通过字典类型dictTypes从缓存中查询字典数据trees
     *
     * @param param
     * @return
     * @throws JsonProcessingException
     */
    @RequestMapping("/getDictTreesByTypes")
    @ResponseBody
    public ResponseBodyVO getDictTreesByTypes(@RequestBody @Validated DictParam param) throws JsonProcessingException {
        ResponseBodyVO<Map<String, List<DictVO>>> response = new ResponseBodyVO<Map<String, List<DictVO>>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(dictService.getDictTreesByTypes(param));
        return response;
    }

    /**
     * 通过字典类型dictTypes从缓存中查询字典数据maps
     *
     * @param param
     * @return
     * @throws JsonProcessingException
     */
    @RequestMapping("/getDictMapsByTypes")
    @ResponseBody
    public ResponseBodyVO getDictMapsByTypes(@RequestBody @Validated DictParam param) throws JsonProcessingException {
        ResponseBodyVO<Map<String, Map<String, String>>> response = new ResponseBodyVO<Map<String, Map<String, String>>>(ResponseBodyVO.Response.SUCCESS);
        response.setInfo(dictService.getDictMapsByTypes(param));
        return response;
    }

}
